@@ -1,5 +1,6 @@
-#![allow(unstable)]
 #![unstable]
+#![feature(core, collections, unicode)]
+#![cfg_attr(test, feature(test))]
 
 //!This crate contains parser combinators, roughly based on the Haskell library [parsec](http://hackage.haskell.org/package/parsec).
 //!
@@ -13,18 +14,19 @@
 //!
 //!```
 //! extern crate "parser-combinators" as parser_combinators;
-//! use parser_combinators::{spaces, chars1, sep_by, digit, satisfy, Parser, ParserExt};
+//! use parser_combinators::{spaces, many1, sep_by, digit, satisfy, Parser, ParserExt, ParseError};
 //! 
 //! fn main() {
 //!     let input = "1234, 45,78";
 //!     let spaces = spaces();
 //!     let integer = spaces.clone()//Parse spaces first and use the with method to only keep the result of the next parser
-//!         .with(chars1(digit()).map(|string| string.parse::<i32>().unwrap()));//parse a string of digits into an i32
+//!         .with(many1(digit()).map(|string: String| string.parse::<i32>().unwrap()));//parse a string of digits into an i32
 //!     //Parse integers separated by commas, skipping whitespace
 //!     let mut integer_list = sep_by(integer, spaces.skip(satisfy(|c| c == ',')));
 //! 
 //!     //Call parse with the input to execute the parser
-//!     match integer_list.parse(input) {
+//!     let result: Result<(Vec<i32>, &str), ParseError> = integer_list.parse(input);
+//!     match result {
 //!         Ok((value, _remaining_input)) => println!("{:?}", value),
 //!         Err(err) => println!("{}", err)
 //!     }
@@ -42,7 +44,7 @@
 //!
 //!```
 //! extern crate "parser-combinators" as parser_combinators;
-//! use parser_combinators::{between, spaces, chars1, sep_by, satisfy, Parser, ParserExt,
+//! use parser_combinators::{between, spaces, many1, sep_by, satisfy, Parser, ParserExt,
 //! ParseResult};
 //! use parser_combinators::primitives::{State, Stream};
 //!
@@ -53,7 +55,7 @@
 //! }
 //! fn expr<I>(input: State<I>) -> ParseResult<Expr, I>
 //!     where I: Stream<Item=char> {
-//!     let word = chars1(satisfy(|c| c.is_alphabetic()));
+//!     let word = many1(satisfy(|c| c.is_alphabetic()));
 //!     let comma_list = sep_by(expr as fn (_) -> _, satisfy(|c| c == ','));
 //!     let array = between(satisfy(|c| c == '['), satisfy(|c| c == ']'), comma_list);
 //!     let spaces = spaces();
@@ -81,21 +83,31 @@ extern crate test;
 #[doc(inline)]
 pub use primitives::{Parser, ParseResult, ParseError};
 #[doc(inline)]
-pub use parser::{
+pub use char::{
     any_char,
+    digit,
+    space,
+    spaces,
+    newline,
+    crlf,
+    tab,
+    upper,
+    lower,
+    letter,
+    alpha_num,
+    hex_digit,
+    oct_digit,
+    string,
+    satisfy,
+};
+#[doc(inline)]
+pub use combinator::{
     between,
     chainl1,
-    chars,
-    chars1,
-    digit,
     many,
     many1,
     optional,
-    satisfy,
     sep_by,
-    space,
-    spaces,
-    string,
     try,
     value,
     unexpected,
@@ -104,10 +116,16 @@ pub use parser::{
     ParserExt
 };
 
+macro_rules! static_fn {
+    (($($arg: pat, $arg_ty: ty),*) -> $ret: ty { $body: expr }) => { { fn temp($($arg: $arg_ty),*) -> $ret { $body } temp } }
+}
+
 ///Module containing the primitive types which is used to create and compose more advanced parsers
 pub mod primitives;
 ///Module containing all specific parsers
-pub mod parser;
+pub mod combinator;
+///Module containg parsers specialized on character streams
+pub mod char;
 
 #[cfg(test)]
 mod tests {
@@ -117,7 +135,7 @@ mod tests {
 
     fn integer<'a, I>(input: State<I>) -> ParseResult<i64, I>
         where I: Stream<Item=char> {
-        let (s, input) = try!(chars1(digit())
+        let (s, input) = try!(many1::<String, _>(digit())
             .parse_state(input));
         let mut n = 0;
         for c in s.chars() {
@@ -145,8 +163,8 @@ mod tests {
     }
     #[test]
     fn field() {
-        let word = chars(satisfy(|c| c.is_alphanumeric()));
-        let word2 = chars(satisfy(|c| c.is_alphanumeric()));
+        let word = many(satisfy(|c| c.is_alphanumeric()));
+        let word2 = many(satisfy(|c| c.is_alphanumeric()));
         let spaces = spaces();
         let c_decl = word
             .skip(spaces.clone())
@@ -173,7 +191,7 @@ r"
         assert_eq!(result, Ok((123i64, state)));
     }
 
-    #[derive(Show, PartialEq)]
+    #[derive(Debug, PartialEq)]
     enum Expr {
         Id(String),
         Int(i64),
@@ -182,7 +200,7 @@ r"
         Times(Box<Expr>, Box<Expr>),
     }
     fn expr(input: State<&str>) -> ParseResult<Expr, &str> {
-        let word = chars1(satisfy(|c| c.is_alphabetic()));
+        let word = many1(satisfy(|c| c.is_alphabetic()));
         let integer = integer as fn (_) -> _;
         let array = between(satisfy(|c| c == '['), satisfy(|c| c == ']'), sep_by(expr as fn (_) -> _, satisfy(|c| c == ',')));
         let spaces = spaces();
@@ -265,7 +283,7 @@ r"
     #[test]
     fn error_position() {
         let mut p = string("let").skip(follow as fn (_) -> _).map(|x| x.to_string())
-            .or(chars1(satisfy(|c| c.is_digit(10))));
+            .or(many1(satisfy(|c| c.is_digit(10))));
         match p.parse("le123") {
             Ok(_) => assert!(false),
             Err(err) => assert_eq!(err.position, SourcePosition { line: 1, column: 1 })
@@ -296,7 +314,7 @@ r"(3 * 4) + 2 * 4 * test + 4 * aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa
 
     #[test]
     fn sep_by_error_consume() {
-        let mut p = sep_by(string("abc"), satisfy(|c| c == ','));
+        let mut p = sep_by::<Vec<_>, _, _>(string("abc"), satisfy(|c| c == ','));
         let err = p.parse("ab,abc")
             .map(|x| format!("{:?}", x))
             .unwrap_err();
@@ -319,7 +337,7 @@ r"(3 * 4) + 2 * 4 * test + 4 * aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa
 
     #[test]
     fn inner_error_consume() {
-        let mut p = many(between(string("["), string("]"), satisfy(|c| c.is_digit(10))));
+        let mut p = many::<Vec<_>, _>(between(string("["), string("]"), satisfy(|c| c.is_digit(10))));
         let result = p.parse("[1][2][]");
         assert!(result.is_err(), format!("{:?}", result));
         let error = result
@@ -330,7 +348,14 @@ r"(3 * 4) + 2 * 4 * test + 4 * aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa
 
     #[test]
     fn infinite_recursion_in_box_parser() {
-        let _ = (many(Box::new(digit())))
+        let _: Result<(Vec<_>, _), _> = (many(Box::new(digit())))
             .parse("1");
     }
+
+    #[test]
+    fn unsized_parser() {
+        let mut parser = Box::new(digit()) as Box<Parser<Input=&str, Output=char>>;
+        let borrow_parser = &mut *parser;
+        assert_eq!(borrow_parser.parse("1"), Ok(('1', "")));
+    }
 }
